/**
 *
 * dialog.cpp
 *
 * Copyright (c) 2008 Douglas Young <rcxdude@gmail.com>
 * Input shaping adapted from FreeWins plugin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 **/

#include "dialog.h"
#include <cmath>

COMPIZ_PLUGIN_20090315 (dialog, DialogPluginVTable);

bool
DialogWindow::shouldFade ()
{
    if (window->overrideRedirect ())
	return false;

    if (!window->isViewable ())
	return false;

    if (!DialogScreen::get (screen)->optionGetFadingWindows ().evaluate (window))
	return false;

    return true;
}

bool
DialogWindow::glPaint (const GLWindowPaintAttrib &attrib,
		       const GLMatrix		 &transform,
		       const CompRegion		 &region,
		       unsigned int		 mask)
{
    if (mFaded)
    {
	DIALOG_SCREEN (screen);
	GLWindowPaintAttrib wAttrib (attrib);
	wAttrib.opacity  *= (ds->optionGetOpacity () / 100);
	wAttrib.saturation *= (ds->optionGetSaturation () / 100);
	wAttrib.brightness *= (ds->optionGetBrightness () / 100);

	return gWindow->glPaint (wAttrib, transform, region, mask);
    }

    return gWindow->glPaint (attrib, transform, region, mask);
}

void
DialogScreen::preparePaint (int ms)
{
    cScreen->preparePaint (ms);
}

void
DialogScreen::handleEvent (XEvent *event)
{
    if (event->type == UnmapNotify)
    {
	CompWindow *w = screen->findWindow (event->xunmap.window);
	if (w && w->transientFor ())
	{
	    CompWindow   *ww  = screen->findWindow (w->transientFor ()); 
	    if (ww)
	    {
		DialogWindow *dww = DialogWindow::get (ww);
		if (dww->mFaded)
		{
		    int othertrans = 0;
		    foreach (CompWindow *www, screen->windows ())
		    {
			if (www->transientFor () == ww->id () && w->id () != www->id () &&
			    optionGetDialogtypes ().evaluate (www))
			    othertrans++;
		    }
		    if (!othertrans)
		    {
			dww->mFaded = false;
			CompositeWindow::get (ww)->addDamage ();
		    }
		}
	    }
	}
    }
    if (event->type == MapNotify)
    {
	CompWindow *w = screen->findWindow (event->xmap.window);
	if (w && w->transientFor () && optionGetDialogtypes ().evaluate (w))
	{
	    CompWindow   *ww  = screen->findWindow (w->transientFor ()); 
	    DialogWindow *dww = DialogWindow::get (ww);
	    if (!dww->mFaded && dww->shouldFade ())
	    {
		dww->mFaded = true;
		dww->cWindow->addDamage ();
	    }
	}
    }

    screen->handleEvent (event);
}

void
DialogScreen::optionChanged (CompOption             *option,
			     DialogOptions::Options num)
{
    foreach (CompWindow *w, screen->windows ())
    {
	DIALOG_WINDOW (w);
	dw->cWindow->addDamage ();
    }
}

DialogScreen::DialogScreen (CompScreen *screen) :
    PluginClassHandler <DialogScreen, CompScreen> (screen),
    cScreen (CompositeScreen::get (screen)),
    gScreen (GLScreen::get (screen))
{
    ScreenInterface::setHandler (screen);
    CompositeScreenInterface::setHandler (cScreen);

    optionSetOpacityNotify (boost::bind (&DialogScreen::optionChanged, this, _1, _2));
    optionSetSaturationNotify (boost::bind (&DialogScreen::optionChanged, this, _1, _2));
    optionSetBrightnessNotify (boost::bind (&DialogScreen::optionChanged, this, _1, _2));
}

DialogWindow::DialogWindow (CompWindow *w) :
    PluginClassHandler <DialogWindow, CompWindow> (w),
    window (w),
    cWindow (CompositeWindow::get (w)),
    gWindow (GLWindow::get (w)),
    mFaded (false),
    mOldTransient (None)
{
    GLWindowInterface::setHandler (gWindow);
}

bool
DialogPluginVTable::init ()
{
    if (!CompPlugin::checkPluginABI ("core", CORE_ABIVERSION) ||
	!CompPlugin::checkPluginABI ("composite", COMPIZ_COMPOSITE_ABI) ||
	!CompPlugin::checkPluginABI ("opengl", COMPIZ_OPENGL_ABI))
	 return false;

    return true;
}
