/**
 * Compiz Peek plugin
 *
 * peek.h
 *
 * Copyright (c) 2008 Sam Spilsbury <smspillaz@gmail.com>
 *
 * Based on opacify.c and thumbnail.c:
 *
 * Copyright (c) 2006 Kristian Lyngstøl <kristian@beryl-project.org>
 * Ported to Compiz and BCOP usage by Danny Baumann <maniac@beryl-project.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Peek is a clone of the Windows 7 'Aero Peeks' feature, which highlights
 * the currently hovered window on the taskbar and dims all the rest. 
 * 
 */

/* Border painting == broken */
#undef USE_BORDER_PAINT

#include <core/core.h>
#include <core/pluginclasshandler.h>

#include <composite/composite.h>
#include <opengl/opengl.h>

#include <mousepoll/mousepoll.h>

#include "peek_options.h"
#include "peek_tex.h"

#define WIN_X(w) ((w)->x () - (w)->input ().left)
#define WIN_Y(w) ((w)->y () - (w)->input ().top)
#define WIN_W(w) ((w)->width () + (w)->input ().left + (w)->input ().right)
#define WIN_H(w) ((w)->height () + (w)->input ().top + (w)->input ().bottom)

class PeekScreen :
    public PluginClassHandler <PeekScreen, CompScreen>,
    public PeekOptions,
    public ScreenInterface,
    public GLScreenInterface
{
    public:

	PeekScreen (CompScreen *);
	~PeekScreen ();

	CompositeScreen *cScreen;
	GLScreen 	*gScreen;

	CompTimer	displayTimeout;
	MousePoller	poller;

	CompWindow	*activeWindow;

#ifdef USE_BORDER_PAINT
	GLTexture::List windowTex;
#endif

	bool		paintingWindowOnTop;

	void
	handleEvent (XEvent *event);

	bool
	glPaintOutput (const GLScreenPaintAttrib &attrib,
		       const GLMatrix            &transform,
		       const CompRegion          &region,
		       CompOutput                *output,
		       unsigned int              mask);

	CompWindow *
	findWindowAt (CompPoint);

	CompWindow *
	findDockAt (CompPoint);

	void
	handleDockEnter ();

	void
	handleDockLeave ();

	void
	positionUpdate (const CompPoint &p);

	bool
	handleTimeout (CompWindow *w);
};

#define PEEK_SCREEN(s)							       \
    PeekScreen *ps = PeekScreen::get (s);

class PeekWindow :
    public PluginClassHandler <PeekWindow, CompWindow>,
    public GLWindowInterface
{
    public:

	PeekWindow (CompWindow *);

	CompWindow 	*window;
	CompositeWindow *cWindow;
	GLWindow   	*gWindow;

	bool peekCandidate;
	bool mTransparent;
	int  mOpacity;
	int  mBrightness;
	int  mSaturation;

	bool
	glPaint (const GLWindowPaintAttrib &attrib,
		 const GLMatrix		   &transform,
		 const CompRegion	   &region,
		 unsigned int		   mask);

	void
	setAttributes (int, int, int);

	bool
	checkPosition ();

	void
	paintBorder ();
};

#define PEEK_WINDOW(w)							       \
    PeekWindow *pw = PeekWindow::get (w);

class PeekPluginVTable :
    public CompPlugin::VTableForScreenAndWindow <PeekScreen, PeekWindow>
{
    public:

	bool init ();
};
