#include "flyingwindows.h"


#define NE 0
#define NO 1
#define SE 2
#define SO 3
#define C 4

void ScreenFlyingWindows::handleEvent( XEvent *event )
{
	ScreenEffect::handleEvent( event );
	if( event->type == MapNotify )
	{
		CompWindow* w = screen->findWindow (event->xmap.window );
		if(w)
			WindowFlyingWindows::getInstance(w).initWindow();
	}
}

bool ScreenFlyingWindows::enable()
{
	ss->mAngleCam = 0.0;
	ss->mScreenCenter = Vector( 0.0, ss->optionGetBounce() ? 0.2 : 0.0,\
								-ss->optionGetAttractionDepth() );
	ss->mCamera = Matrix::identity;
	ss->mDesktopOpacity = OPAQUE;

	foreach (CompWindow *w, screen->windows ())
		WindowFlyingWindows::getInstance(w).initWindow();
	
	return ScreenEffect::enable();
}

void ScreenFlyingWindows::disable()
{
	foreach (CompWindow *w, screen->windows ())
	{
		WindowFlyingWindows& sw = WindowFlyingWindows::getInstance(w);
		if( sw.active )
			sw.transformFadeOut = ss->mCameraMat * sw.transform;

		else sw.opacityFadeOut = sw.opacity;
	}
	ss->mCameraMat = Matrix::identity;
	ScreenEffect::disable();
}

void ScreenFlyingWindows::addForce( const Point& p1, const Point& p2, const Point& center, Vector& resultante, Vector& couple, float w, bool attract )
{	
	Vector u = p2 - p1;
	float d = u.norm();
	u.normalize();
	
	if( d < 1e-5 ) d = 1e-5;

	Vector force = attract ? w*u*d*d : -w*u/(d*d);
	resultante += force;
	
	couple += ( center - p1 ) ^ force;
}

void ScreenFlyingWindows::preparePaint( int msSinceLastPaint )
{
	ScreenEffect::preparePaint ( msSinceLastPaint );
	
	float ratio = ss->optionGetAttractionRepulsionRatio()/100.0;
	float wAt = ratio;
	float wRt = 1-ratio;
	
	if( ss->mState.fadingIn )
	{
		wAt *= getProgress();
		wRt *= getProgress();
		
		ss->mDesktopOpacity = (GLushort)( OPAQUE * ( 1.0 - getProgress() ));
	}
	
	if( ss->mState.fadingOut )
	{	
		ss->mDesktopOpacity = (GLushort)(OPAQUE * getProgress());
	}

	if( !ss->mState.fadingOut )
	{
		ss->mAngleCam += ((float)msSinceLastPaint)/100000.0;
		if( ss->mAngleCam > 0.03 ) ss->mAngleCam=0.03;
		
		ss->mCamera.rotate( ss->mAngleCam*msSinceLastPaint, 0.0, 1.0, 0.0 );

		Matrix centerTrans = Matrix::identity;
		Matrix centerTransInv = Matrix::identity;
		
		Vector screenCenterTranslated = ss->mScreenCenter.toCoordsSpace();
		screenCenterTranslated[z] *= screen->width ();
		
		centerTrans.scale( 1.0, 1.0, 1.0 / screen->width ());
		centerTrans.translate( screenCenterTranslated );
		centerTransInv.translate( -screenCenterTranslated );
		centerTransInv.scale( 1.0, 1.0, 1.0 * screen->width ());
		
		ss->mCameraMat = centerTrans * ss->mCamera * centerTransInv;
	}
	
	foreach (CompWindow *w, screen->windows ())
	{
		WindowFlyingWindows& sw = WindowFlyingWindows::getInstance(w);

		if( sw.active )
		{
			if( ss->mState.fadingOut )
				sw.transform = interpolate( sw.transformFadeOut, Matrix::identity, getProgress() );

			else
			{
				int collisionVertex = -1;
				int collisionIteration = 1;
				Vector a, p, o, omega;
				do
				{
					Vector resultante, couple, resultanteA, coupleA;
					resultante = couple = resultanteA = coupleA = Vector::null;
					Vector windowcenter = sw.vertex[C];
					float mass = sqrt(((float)(WIN_W(w)*WIN_H(w)))/(screen->width () * screen->height ()));
					
					float wR = 1e-8/mass*wRt;
					float wA = 1e-8/mass*wAt;
					
					int numPoint = 0;
					for( int i = 0; i<5; i++)
					{
						foreach (CompWindow *w2, screen->windows ())
						{	
							WindowFlyingWindows& sw2 = WindowFlyingWindows::getInstance(w2);
							if( w2 != w && sw2.active )
							{
								numPoint++;
								for( int j = 0; j<5; j++)
									addForce( sw.vertex[i],  sw2.vertex[j], windowcenter, resultante, couple, wR, FALSE );
							}
						}
						addForce( sw.vertex[i], ss->mScreenCenter, windowcenter, resultanteA, coupleA, wA, TRUE );
					}
					
					if( numPoint < 1 )
						numPoint = 1;
						
					resultante += resultanteA*numPoint;
					couple += coupleA*numPoint;

					if( collisionVertex != -1 )
					{
						float wb = sw.vertex[collisionVertex][y]/msSinceLastPaint*5e-4;
						resultante[y] = -wb;
						float tmp = couple[z];
						couple = (sw.vertex[collisionVertex]-windowcenter) ^ Vector( 0.0, -wb, 0.0 );
						couple[z] = tmp;
						sw.speed[y] = 0;
					}

					a = resultante - 5e-4*mass*sw.speed;
					omega = couple*1000 - 5e-3*mass*sw.speedrot;

					p = msSinceLastPaint*msSinceLastPaint*a+msSinceLastPaint*sw.speed;
					sw.speed += msSinceLastPaint*a;
	
					o = msSinceLastPaint*msSinceLastPaint*omega+msSinceLastPaint*sw.speedrot;
					sw.speedrot += msSinceLastPaint*omega;
					
					sw.transformTrans.translate( p[x]*screen->width (), -p[y]*screen->height (), p[z] );
					sw.transformRot.rotate( o.norm(), o );
					
					sw.transform = sw.transformTrans * sw.centerTrans * sw.transformRot * sw.centerTransInv;
					
					sw.recalcVertices();
					
					if ( ss->optionGetBounce () )
					{
						for( int i = 0; i < 4; i++ )
							if( sw.vertex[i][y] < -0.5 )
								collisionVertex = i;
					}
					
				} while ( collisionVertex != -1 && collisionIteration-- );
			}
		}
		else
		{
			if( ss->mState.fadingOut )
			{
				sw.opacity = (GLushort)( sw.opacityOld*getProgress() + sw.opacityFadeOut*(1-getProgress()) );
			}
			else
			sw.steps = (int)( (msSinceLastPaint * OPAQUE) / (ss->optionGetFadeInDuration()*1000.0) );
		}
	}
}

void ScreenFlyingWindows::donePaint ()
{
	ss->cScreen->damageScreen();
	ScreenEffect::donePaint ();
}

void ScreenFlyingWindows::paint (CompOutput::ptrList &outputs,
				 unsigned int	     mask)
{
    CompOutput::ptrList newOutputs;
    newOutputs.push_back (&screen->fullscreenOutput ());

    ScreenEffect::paint (newOutputs, mask);
}

bool
ScreenFlyingWindows::glPaintOutput(const GLScreenPaintAttrib &attrib,
				   const GLMatrix	     &transform,
				   const CompRegion	     &region,
				   CompOutput		     *output,
				   unsigned int		     mask)
{
	ss->gScreen->clearTargetOutput (GL_COLOR_BUFFER_BIT);
	return ScreenEffect::glPaintOutput( attrib, transform, region, output, mask | PAINT_SCREEN_TRANSFORMED_MASK );
}

void
ScreenFlyingWindows::glPaintTransformedOutput (const GLScreenPaintAttrib &attrib,
					       const GLMatrix		 &transform,
					       const CompRegion		 &region,
					       CompOutput		 *output,
					       unsigned int		 mask)
{
	bool wasCulled = glIsEnabled (GL_CULL_FACE);
	if (wasCulled)
		glDisable (GL_CULL_FACE);

	GLenum oldFilter = ss->gScreen->textureFilter ();
	if( ss->optionGetMipmaps() )
		ss->gScreen->setTextureFilter (GL_LINEAR_MIPMAP_LINEAR);
	
	
	mask &= ~PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_MASK;

	GLboolean bTwoSite;
	glGetBooleanv(GL_LIGHT_MODEL_TWO_SIDE, &bTwoSite );
	glLightModeli (GL_LIGHT_MODEL_TWO_SIDE, ss->gScreen->lighting ());
	
	ScreenEffect::glPaintTransformedOutput (attrib, transform, region, output, mask);

	glLightModeli (GL_LIGHT_MODEL_TWO_SIDE, bTwoSite);

	ss->gScreen->setFilter (SCREEN_TRANS_FILTER, GLTexture::Good);
	ss->gScreen->setTextureFilter (oldFilter);

	if (wasCulled)
		glEnable (GL_CULL_FACE);
}

WindowFlyingWindows::WindowFlyingWindows( CompWindow* w ) :
	WindowEffect(w),
	active( FALSE ),
	opacity( GLWindow::get (w)->paintAttrib ().opacity ),
	opacityFadeOut( 0 ),
	opacityOld( 0 ),
	steps(0)
{

}

// Returns true if w is a flying window
bool WindowFlyingWindows::isActiveWin()
{
	return	!w->overrideRedirect () && \
		 w->mapNum () && \
		 w->isViewable () && \
		 !( w->wmType () & ( CompWindowTypeDockMask | CompWindowTypeDesktopMask )) && \
		/*	!( w->state & ( CompWindowStateSkipPagerMask | CompWindowStateShadedMask )) && \*/
		 ScreenSaverScreen::get (screen)->optionGetWindowMatch ().evaluate (w);
}

// Initialize window transformation matrices and vertices
void WindowFlyingWindows::initWindow()
{	
	active = isActiveWin();
	if( active )
	{
		float x = WIN_X(this->w);
		float y = WIN_Y(this->w);
		float w = WIN_W(this->w);
		float h = WIN_H(this->w);
		
		transform = transformRot = transformTrans = Matrix::identity;
		centerTrans = centerTransInv = Matrix::identity;
		
		centerTrans.scale( 1.0, 1.0, 1.0/screen->width () );
		centerTrans.translate( x + w/2.0, y + h/2.0, 0.0 );
		centerTransInv.translate( -(x + w/2.0), -(y + h/2.0), 0.0);
		centerTransInv.scale( 1.0f, 1.0f, 1.0f*screen->width ()  );
	
		recalcVertices();
		speed = speedrot = Vector::null;
	}
	else opacityOld = opacity;
}

// Update window vertices
void WindowFlyingWindows::recalcVertices()
{
	float x = WIN_X(this->w);
	float y = WIN_Y(this->w);
	float w = WIN_W(this->w);
	float h = WIN_H(this->w);

	vertex[NO] = Point( x, y, 0.0 );
	vertex[NE] = Point( x + w, y, 0.0 );
	vertex[SO] = Point( x, y + h, 0.0 );
	vertex[SE] = Point( x + w, y + h, 0.0 );
	vertex[C] = Point( x + w/2.0, y + h/2.0, 0.0 );

	// Apply the window transformation and normalize
	for( int i = 0; i < 5; i++ )
		vertex[i] = ( transform * vertex[i] ).toScreenSpace();
}

bool WindowFlyingWindows::glPaint (const GLWindowPaintAttrib &attrib,
				   const GLMatrix	     &transform,
				   const CompRegion	     &region,
				   unsigned int		     mask)
{
	GLWindowPaintAttrib sAttrib (attrib);
	Matrix wTransform;
	
	if( !active )
	{
		SCREENSAVER_SCREEN (screen);
		
		if( opacity && steps && !ss->mState.fadingOut )
		{
			if( opacity < steps )
				opacity = 0;
			else opacity -= steps;
			steps = 0;
		}
			
		sAttrib.opacity = opacity;
		wTransform = transform;
	}
	else
	{
		SCREENSAVER_SCREEN (screen);
		wTransform = transform * ss->mCameraMat * this->transform;
		mask |= PAINT_WINDOW_TRANSFORMED_MASK;
		
		if( w->state () & CompWindowStateSkipPagerMask )
			return WindowEffect::glPaint (sAttrib, wTransform.m, region, mask );
	}
	
	if (w->alpha () || sAttrib.opacity != OPAQUE)
		mask |= PAINT_WINDOW_TRANSLUCENT_MASK;
	
	// from paint.cpp
	if (mask & PAINT_WINDOW_OCCLUSION_DETECTION_MASK)
	{
		if (mask & PAINT_WINDOW_TRANSFORMED_MASK)
			return FALSE;
		if (mask & PAINT_WINDOW_NO_CORE_INSTANCE_MASK)
			return FALSE;
		if (mask & PAINT_WINDOW_TRANSLUCENT_MASK)
			return FALSE;
		if (w->shaded ())
			return FALSE;
		return TRUE;
	}
	
	GLFragment::Attrib fragment (sAttrib);
	
	glPushMatrix();
	glLoadMatrixf( wTransform.m );
	bool status = sw->gWindow->glDraw (wTransform.m, fragment, region, mask);
	glPopMatrix();
	return status;
}
