/*
 * Compiz Static Windows plugin
 *
 * static.h
 *
 * Copyright : (C) 2008 by Mark Thomas
 * E-mail    : markbt@efaref.net
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <core/core.h>
#include <core/pluginclasshandler.h>
#include <opengl/opengl.h>
#include <composite/composite.h>

#include "static_options.h"

typedef enum _StaticMode
{
    STATIC_ALL,
    STATIC_NORMAL,
    STATIC_STATIC
}
StaticMode;

class StaticScreen :
    public PluginClassHandler <StaticScreen, CompScreen>,
    public GLScreenInterface,
    public CompositeScreenInterface,
    public StaticOptions
{
    public:

	StaticScreen (CompScreen *);
	~StaticScreen ();

    public:

	CompositeScreen *cScreen;
	GLScreen	*gScreen;

    public:

	bool
	glPaintOutput (const GLScreenPaintAttrib &,
		       const GLMatrix		 &,
		       const CompRegion		 &,
		       CompOutput	 	 *,
		       unsigned int		   );

	void
	glApplyTransform (const GLScreenPaintAttrib &,
			  CompOutput		    *,
			  GLMatrix		    *);
	
	void
	glPaintTransformedOutput (const GLScreenPaintAttrib &,
			          const GLMatrix	    &,
			          const CompRegion	    &,
			          CompOutput	 	    *,
			          unsigned int		      );

	void
	preparePaint (int);

    public:

	StaticMode mStaticMode;
	bool	   mPaintingStatic;
};

#define STATIC_SCREEN(s)						       \
    StaticScreen *ss = StaticScreen::get (s);

class StaticWindow :
    public PluginClassHandler <StaticWindow, CompWindow>,
    public GLWindowInterface
{
    public:

	StaticWindow (CompWindow *);
	~StaticWindow ();

    public:

	CompWindow	*window;
	CompositeWindow *cWindow;
	GLWindow	*gWindow;

    public:

	bool
	glPaint (const GLWindowPaintAttrib &,
		 const GLMatrix		   &,
		 const CompRegion	   &,
		 unsigned int		     );

    public:

	bool
	isStatic ();

};

#define STATIC_WINDOW(w)						       \
    StaticWindow *sw = StaticWindow::get (w)

class StaticPluginVTable :
    public CompPlugin::VTableForScreenAndWindow <StaticScreen, StaticWindow>
{
    public:

	bool init ();
};
